/*
 * Title:        CloudSim Toolkit
 * Description:  CloudSim (Cloud Simulation) Toolkit for Modeling and Simulation of Clouds
 * Licence:      GPL - http://www.gnu.org/copyleft/gpl.html
 *
 * Copyright (c) 2009-2012, The University of Melbourne, Australia
 */

package org.cloudbus.cloudsim.checkpoint;

import java.io.BufferedReader;
import java.io.FileReader;
import java.io.IOException;
import java.util.StringTokenizer;

import org.cloudbus.cloudsim.CKP.datacenter.NetworkHost;
import org.cloudbus.cloudsim.core.CloudSim;

/**
 * This class is just an file-reader for the special brite-format! the brite-file is structured as
 * followed: Node-section: NodeID, xpos, ypos, indegree, outdegree, ASid, type(router/AS)
 * Edge-section: EdgeID, fromNode, toNode, euclideanLength, linkDelay, linkBandwith, AS_from, AS_to,
 * type
 * 
 * @author Thomas Hohnstein
 * @since CloudSim Toolkit 1.0
 */
public class FailureEventReader {

	/**
	 * this method just reads the file and creates an TopologicalGraph object
	 * 
	 * @param filename name of the file to read
	 * @return created TopologicalGraph
	 * @throws IOException
	 */
	public static void readFailureEventFile(String filename, DatacenterDestroyer destroyer) throws IOException {

		// lets read the file
		FileReader fr = new FileReader(filename);
		BufferedReader br = new BufferedReader(fr);
		String nextLine = null;

		
		int hostid;
		String eventType;
		double timeOfOccurrence;
		String failureType;
		String failureParameter;
		double mean;
		double shape,scale;
		
		int failureFrequency;
		double lastFailureTime;

		while ((nextLine = br.readLine()) != null) {
			if(nextLine.startsWith("//"))
				continue;
			
			StringTokenizer tokenizer = new StringTokenizer(nextLine);
			
			if(tokenizer.hasMoreElements()){
				String token = tokenizer.nextToken();
				hostid = Integer.valueOf(token);
				failureFrequency = Integer.valueOf(tokenizer.nextToken());
				lastFailureTime = Double.valueOf(tokenizer.nextToken());
				failureType = tokenizer.nextToken();
				failureParameter = tokenizer.nextToken();

				NetworkHost host = (NetworkHost)CloudSim.getEntity(hostid);
				host.failureFrequency = failureFrequency;
				host.lastFailureTime = lastFailureTime;
				if(failureType.equals("Exponential")){
					mean = Double.valueOf(failureParameter);				
					host.failureDistri = new ExponentialFailureDistr(mean);
				}
				while(tokenizer.hasMoreElements()){
					eventType = tokenizer.nextToken();
					timeOfOccurrence = Double.valueOf(tokenizer.nextToken());
					if(eventType.equals("F")){
						destroyer.addFailureEvent(hostid, timeOfOccurrence);
					}
					else{
						destroyer.addRepairEvent(hostid, timeOfOccurrence);
					}
				}
			}
		}
		br.close();
	}

}
